/**
 * Copyright (c) 2013-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.so.api;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import org.junit.Test;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.DuplicatedSubscriptionObjectException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.InvalidSubscriptionObjectNameException;

public class SubscriptionObjectRegistrationTestCase {

    private class SubscriptionObjectRegistrorTest implements SubscriptionObjectRegistror {

        private final Map<String, SubscriptionObject> subscriptionObjects = new HashMap<String, SubscriptionObject>();

        @Override
        public void registersSubscriptionObjects(final SubscriptionObject subscriptionObject)
                throws DuplicatedSubscriptionObjectException, InvalidSubscriptionObjectNameException,
                IllegalArgumentException {
            if (subscriptionObject.getName() == null) {
                throw new InvalidSubscriptionObjectNameException(subscriptionObject);
            } else if (subscriptionObject.getName().equals("duplicated")) {
                throw new DuplicatedSubscriptionObjectException(subscriptionObject);
            } else {
                this.subscriptionObjects.put(subscriptionObject.getName(), subscriptionObject);
            }
        }

        @Override
        public Map<String, SubscriptionObject> getSubscriptionObjects() {
            return this.subscriptionObjects;
        }

        @Override
        public Shell getShell() {
            return null;
        }

    };

    /**
     * Check the automatic registration of subscription objects
     */
    @Test
    public void testRegistersSubscriptionObjects_00() {

        final SubscriptionObjectRegistror subscriptionObjRegistrator = new SubscriptionObjectRegistrorTest();
        SubscriptionObjectRegistration.registers(subscriptionObjRegistrator);
        final SubscriptionObject dummySubscriptionObject = new DummySubscriptionObject();

        final Map<String, SubscriptionObject> subscriptionObjects = subscriptionObjRegistrator
                .getSubscriptionObjects();
        assertEquals("Too much subscription objects registered", 1, subscriptionObjects.size());
        assertEquals("Unexpected registered subscription object",
                dummySubscriptionObject.getName(),
                subscriptionObjects.get(dummySubscriptionObject.getName()).getName());
    }

    /**
     * Check the registration of subscription object having no name (name =
     * <code>null</code>)
     */
    @Test
    public void testRegistersSubscriptionObjects_01() {

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals("META-INF/services/" + SubscriptionObject.class.getName())) {
                    return super.getResources("spi-null");
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            final SubscriptionObjectRegistror subscriptionObjRegistrator = new SubscriptionObjectRegistrorTest();
            SubscriptionObjectRegistration.registers(subscriptionObjRegistrator);
            final Map<String, SubscriptionObject> subscriptionObjects = subscriptionObjRegistrator
                    .getSubscriptionObjects();
            assertEquals("Too much subscription objects registered", 0, subscriptionObjects.size());
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }

    /**
     * Check the registration of a subscription object already registered
     */
    @Test
    public void testRegistersSubscriptionObjects_02() {

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals("META-INF/services/" + SubscriptionObject.class.getName())) {
                    return super.getResources("spi-duplicated");
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            final SubscriptionObjectRegistror subscriptionObjRegistrator = new SubscriptionObjectRegistrorTest();
            SubscriptionObjectRegistration.registers(subscriptionObjRegistrator);
            final Map<String, SubscriptionObject> subscriptionObjects = subscriptionObjRegistrator
                    .getSubscriptionObjects();

            final SubscriptionObject dummySubscriptionObject = new DummySubscriptionObject();
            assertEquals("Too much subscription objects registered", 1, subscriptionObjects.size());
            assertEquals("Unexpected registered subscription object",
                    dummySubscriptionObject.getName(),
                    subscriptionObjects.get(dummySubscriptionObject.getName()).getName());
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }
}
